# use Math::BigFloat;

use DBI;

package xData;

require Exporter;
@EXPORT = qw(six_digits);

use xMisc;

use strict;


my $conf = $main::conf;
unless ($conf)
{
  my ($package, $filename, $line) = caller;
  # warn "getting conf myself, thanks $filename ($line)";
  require Conf;
  $conf = new Conf;
  $conf->read('/usr/home/tstock/bin/x-rates.v2.conf') || die;
}
$main::conf = $conf;

my $dbh;


sub one_day
{
  my ($base, $date_in, $value) = @_;
  $base ||= 'USD';
  my $base_v = $base;
  $base_v = 1 if ($base eq 'USD');

  $dbh ||= DBI->connect(
		      "DBI:mysql:$conf->{sql_database}:$conf->{sql_server}",
		      "$conf->{sql_user}",
		      "$conf->{sql_password}",
		      {
		       PrintError => 1,
		       RaiseError => 1
		      }
		     );
  die unless $dbh;

  my $q = "SELECT * FROM rates WHERE $base_v > 0 ORDER BY day DESC LIMIT 1";
  my $sth = $dbh->prepare($q);
  unless ($sth && $sth->execute)
  {
    $@ = "SQL statement error: $dbh->errstr";
    return undef;
  }

  my $results = $sth->fetchrow_hashref;
  $sth->finish;
#  $dbh->disconnect;

  $results || die "no values for this currency in the database";

  $results->{USD} = 1;

  my $date_out = $results->{day} || die;
  my $base_val = $results->{$base} || die;

  delete $results->{day};
  delete $results->{$base};

  my $ret = {};

  $base_val || die "[$base_val] not good base";
  for (keys %$results)
  {
    next unless $results->{$_};
    $ret->{$_} = $results->{$_} / $base_val;
  }

  {
    # some countries just arent ready for this much talent...
 
    #delete $ret->{PHP};
    #delete $ret->{IDR};
    delete $ret->{BDT};

    # deleted in 2004
    #delete $ret->{BHD};
    #delete $ret->{CZK};
    #delete $ret->{AED};
   # delete $ret->{PLN};
   # delete $ret->{IQD};
#
    # euro
    delete $ret->{BEF};
    delete $ret->{DEM};
    delete $ret->{GRD};
    delete $ret->{ESP};
    delete $ret->{FRF};
    delete $ret->{IEP};
    delete $ret->{ITL};
    delete $ret->{LUF};
    delete $ret->{NLG};
    delete $ret->{ATS};
    delete $ret->{PTE};
    delete $ret->{FIM};
  }


  return ($ret, $date_out, $value);
}


sub two_days
{
  my ($day) = @_;
  # $curr should be array of arrays (pairs of ISO 4217 codes)

  my $wday = ($day) ? $day : '';

  # get all values, last two days
  $dbh ||= DBI->connect(
			 "DBI:mysql:$conf->{sql_database}:$conf->{sql_server}",
			 "$conf->{sql_user}",
			 "$conf->{sql_password}",
			 {
			  PrintError => 1,
			  RaiseError => 1
			 }
			);

  my $q = "SELECT * FROM rates $wday ORDER BY day DESC LIMIT 2";

  my $sth = $dbh->prepare($q);
  unless ($sth && $sth->execute)
  {
    $@ = "SQL statement error: $dbh->errstr";
    return undef;
  }

  my $today = $sth->fetchrow_hashref;
  my $yesterday = $sth->fetchrow_hashref;
  $today->{USD} = $yesterday->{USD} = 1;

  my $dt = {};
  for my $k (keys %$today)
  {
    if ($today->{$k} && $yesterday->{$k})
    {
      $dt->{$k}{today} = $today->{$k};
      $dt->{$k}{yesterday} = $yesterday->{$k};
    }
  }

  $sth->finish;
#  $dbh->disconnect;

  return $dt
}



sub thirty_days
{
  x_days(@_,30) || undef;
}



sub x_days
{
  my ($base, $target, $days) = @_;
#  my ($base, $target, $date_in, $value, $days) = @_;
#  warn "getting data for $days";

  $target = '1' if ($target eq 'USD');
  $base   = '1' if ($base eq 'USD');

  $dbh ||= DBI->connect(
			"DBI:mysql:$conf->{sql_database}:$conf->{sql_server}",
			"$conf->{sql_user}",
			"$conf->{sql_password}",
		      {
		       PrintError => 1,
		       RaiseError => 1
		      }
		     );
  die unless $dbh;

  my $q = "SELECT day,$target/$base as val FROM rates ";
  $q .= "WHERE $target/$base > 0 ORDER BY day DESC LIMIT $days";

  my $sth = $dbh->prepare($q);
  unless ($sth && $sth->execute)
  {
    $@ = "SQL statement error: $dbh->errstr";
    return undef;
  }

  my @data;
  while (my $res = $sth->fetchrow_hashref)
  {
    $res->{val} = six_digits($res->{val});
    unshift(@data, $res );
  }

  $sth->finish;
#  $dbh->disconnect;

  return \@data;
}


my %country_cache;
sub country_list
{
  unless (keys %country_cache)
  {
    %country_cache = _get_country_list();
  }

  return %country_cache;
}


sub _get_country_list
{
  $dbh ||= DBI->connect(
		      "DBI:mysql:$conf->{sql_database}:$conf->{sql_server}",
		      "$conf->{sql_user}",
		      "$conf->{sql_password}",
		      {
		       PrintError => 1,
		       RaiseError => 1
		      }
		     );
  die unless $dbh;

  my $q = "SELECT * FROM rates ORDER BY day DESC LIMIT 1";

  my $sth = $dbh->prepare($q);
  unless ($sth && $sth->execute)
  {
    $@ = "SQL statement error: $dbh->errstr";
    return undef;
  }

  my $results = $sth->fetchrow_hashref;
  $results->{USD} = 1;

  $q = "SELECT currency FROM country_info WHERE iso4217=?";
  $sth = $dbh->prepare($q);

  my %cc;
  for my $c (keys %$results)
  {
    $sth->execute($c);
    my @a = $sth->fetchrow;
    $cc{$c} = $a[0] if $a[0];
#    `echo $a[0] >> /tmp/lala` if $a[0];
  }

  $sth->finish;
#  $dbh->disconnect;

  {
    # some countries just arent ready for this much talent...
    #delete $cc{PHP};
    #delete $cc{IDR};
    delete $cc{BDT};

    # deleted in 2004
    #delete $cc{BHD};
    #delete $cc{CZK};
    #delete $cc{AED};
    #delete $cc{PLN};
    #delete $cc{IQD};

    # euro
    delete $cc{BEF};
    delete $cc{DEM};
    delete $cc{GRD};
    delete $cc{ESP};
    delete $cc{FRF};
    delete $cc{IEP};
    delete $cc{ITL};
    delete $cc{LUF};
    delete $cc{NLG};
    delete $cc{ATS};
    delete $cc{PTE};
    delete $cc{FIM};
  }

  return %cc;
}


sub six_digits
{
  # Thoroughly benchmarked, sprintf %6g is not faster.
  # however, it does round the last decimal number...
  # maybe I should use it, taking a 30% hit in speed
  # everytime this function is called, which is often

  my $val = shift;

  if (($val == int($val)) || ($val > 99999)) # mostly an integer
  {
    return int($val);
  }
  else
  {
    return sprintf("%.6g",$val);

    if ( $val > 1 ) # 0.x or x.x or integer with 5- digits
    {
      return 0 + substr($val, 0, 7)
    } else {
      return 0 + substr($val, 0, 8);
    }
  }
}

1;
